CON
    _clkmode = xtal1 + pll16x                           
    _xinfreq = 5_000_000

    #0,MSBPRE,LSBPRE,MSBPOST,LSBPOST                                            'Used for SHIFTIN routines
    #4,LSBFIRST,MSBFIRST                                                        'Used for SHIFTOUT routines
    #12,Dpin,Cpin,Latch,SLpin,#16,Bits                                                 'Set Dpin,Cpin,SLpin and Bit constant
    
VAR     word          DataValue
        word          pins
        word          stack[100]
        word          Array1[50],Array2[50],Array3[50],Array4[50],Array5[50]
        byte          Timer 
OBJ     SPI     : "Controller engine"
        SYS     : "SPI Engine"
        Ser     : "FullDuplexSerial"

PUB start
{
                        Once called from Spin, SHIFTIN or SHIFTOUT remains running in its own COG.
                        If SHIFTIN or SHIFTOUT are called with 'Bits' set to Zero, then the COG will shut
                        down.  Another way to shut the COG down is to call 'stop' from Spin.
}          
{
    PIN to BUTTON LAYOUT
    pin# | button        | In/Out
    ----------------------------
    0    | B             | IN
    1    | Y             | IN
    2    | select        | IN
    3    | start         | IN
    4    | up            | IN
    5    | down          | IN
    6    | left          | IN
    7    | right         | IN
    8    | A             | IN
    9    | X             | IN
    10   | L             | IN
    11   | R             | IN
    12   | Data Out      | OUT
    13   | System Clock  | IN
    14   | Latch         | IN
    15   | Rapid Fire    | IN
    16   | macro 1       | IN
    17   | Macro 2       | IN
    18   | Macro 3       | IN
    19   | Macro 4       | IN
    20   | Macro 5       | IN
}

      
'------------------------------------------------------------------------------------------------------------------------------
    cognew(The_system,@stack)                                          'settes up a a cog to act as the system for testing purposes
    cognew(time,@stack)
      ser.start(31,30,0,9600) 

    dira[0..13]~                                                       'settes up the button inputs for the main 12, the latch, and the clock
    dira[14]~~                                                         'settes up the main data output
    dira[16..20]~                                                      'settes up the array buttons
    datavalue := ina[0..11]                                            'sends out the first button scan for output
    SPI.SHIFTOUT(Dpin, Cpin, MSBFIRST, Bits, datavalue,Latch)
    repeat                                                             'The master loop
      datavalue := ina[0..11]                                          'the regular button scan
      if !ina[15] == 1                                                 'check if the rapid fire is pressed
        rapid
      elseif !ina[16..20] and %111111                                  'check if a array button is pressed
        case ina[16..20]                                               'goes through and see which array was pressed and act accordenly
          %01111: Array(@Array1,16)
          %10111: Array(@Array2,17)
          %11011: Array(@Array3,18)
          %11101: Array(@Array4,19)
          %11110: Array(@Array5,20)
        datavalue := ina[0..11]
      repeat while timer == 0
      SPI.SHIFTOUT(Dpin, Cpin, MSBFIRST, Bits, datavalue,Latch)
    repeat
                               'Send SHIFTOUT DataVAlue
      ser.dec(200)
      ser.tx(13)
      ser.tx(10)   
                                                                                   'DEBUG - Tested with 74HCT164
                                                                                '        for MSBFIRST and LSBFIRST functions
{------------------------------------------------------------------------------------------------------------------------------
    dira[23..16] := %11111111                                                   'DEBUG - Make ALL LED's outputs

    dira[SLpin] := %1                                                           'Set Shift / Load pin as an output
    repeat
      outa[SLpin] := %1                                                         'Set Shift / Load pin in shift mode "1"
      DataValue := SPI.SHIFTIN(Dpin, Cpin, MSBPRE, Bits)                        'Get SHIFTIN DataValue
      outa[SLpin] := %0                                                         'Set Shift / Load pin in load mode "0"
      
      outa[23..16] := DataValue                                                 'DEBUG - Lightup the LED's corresponding to the data
                                                                                '        LED23 = MSB
                                                                                '        LED16 = LSB
'------------------------------------------------------------------------------------------------------------------------------
}
{{
                                                                                
Notes:
Consecutive SHIFTIN/SHIFTOUT updates through Spin at 80 MHz is about 14kHz (Bit rate about 112k) 

Consecutive SHIFTIN/SHIFTOUT updates through Assembly at 80 MHz is about 310kHz (Bit rate about 2.5M)

}}
{
This is the rapid fire section where it will toggle pushed buttons every scan
to where it can toggle about 6 times per second
}
pri rapid | last_press
  Last_press := ina[0..11]
  repeat while ina[15] == 0
    last_press := Last_press ^= !ina[0..11]
    repeat while timer == 0
    SPI.SHIFTOUT(Dpin, Cpin, MSBFIRST, Bits, last_press, Latch)

{
This part were if the macro button were press and let go it will
take the macro and play it back until it see all zeros and send it back
to the main loop
if press for approxamatly 3 seconds it will go into programming mode
for that certain array and it will stop transmitting data and supposedly
" unplug " from the system untill the programming is done.
}
pri array(macro,pin) | count, num                                     'brings the array into the 
  repeat while !ina[pin] == 1                                         'for as long as the selected macro button is held down it will keep repeating
    count++                                                           'the incrementing counter
    if count == 200000                                                'when the count reaches the limit the controller will go in to program mode for that macro
      program(@macro)
      return                                                          'when done programming macro return to master loop
  repeat while macro[num] == true                                     'should the array had have button pushes recorded it will play back until it done
    repeat while timer == 0
    SPI.SHIFTOUT(Dpin, Cpin, MSBFIRST, Bits, macro[num++],Latch)
{
This part of the program in theroy the arrays name would be passed down
to where this part would be able to program the array with button pushes
that will be store when the select button is pressed it will stop programing
and it would jump back to the original loop.
}    
pri program(macro) | count, starting_value
  starting_value := %1111_1111_1111                     'sets up the orignial bin number
  repeat while ina[3] == 1                              'as long as the select button is not pressed it will keep recording
    macro[count] := starting_value and ina[0..12]       'when a button is pressed it will change the pattern
    if !ina[15] == 1                                    'when rapid fire is pressed it will go to the next button sequence
      count++                                           'increment the array
      starting_value := %1111_1111_1111                 'resart the number for next button sequence
      repeat while !ina[15] == 0                        'to keep it from skipping to other parts of the array with an extented push of the next record
      
pri The_System | Data_in
  dira[Latch]~~
  outa[Latch]~
  repeat
    waitcnt(clkfreq + cnt)  
    outa[Latch]~~
    Data_in := sys.shiftin(dpin, cpin, Msbpre, bits)
    outa[Latch]~
   
{
This is the master timer it will make sure that data being
send don't overlap each other and make 
}
pri Time                                   
  repeat                                   
     waitpeq(1, 5, 0)                      
     waitcnt(clkfreq / 1000 * 11 + cnt) 
     timer := 1                            
     timer := 0